@@ -1,9 +1,8 @@
 /* pam_group module */
 
 /*
- * $Id: pam_group.c,v 1.19 2006/02/10 18:33:58 kukuk Exp $
- *
  * Written by Andrew Morgan <morgan@linux.kernel.org> 1996/7/6
+ * Field parsing rewritten by Tomas Mraz <tm@t8m.info>
  */
 
 #include "config.h"
@@ -27,14 +26,13 @@
 #define PAM_GROUP_BUFLEN        1000
 #define FIELD_SEPARATOR         ';'   /* this is new as of .02 */
 
-#ifdef TRUE
-# undef TRUE
+#ifndef TRUE
+# define TRUE 1
 #endif
-#ifdef FALSE
-# undef FALSE
+#ifndef FALSE
+# define FALSE 0
 #endif
 
-typedef enum { FALSE, TRUE } boolean;
 typedef enum { AND, OR } operator;
 
 /*
@@ -53,166 +51,156 @@ typedef enum { AND, OR } operator;
 
 /* --- static functions for checking whether the user should be let in --- */
 
-static void shift_bytes(char *mem, int from, int by)
+static char *
+shift_buf(char *mem, int from)
 {
-    while (by-- > 0) {
-	*mem = mem[from];
+    char *start = mem;
+    while ((*mem = mem[from]) != '\0')
 	++mem;
+    memset(mem, '\0', PAM_GROUP_BUFLEN - (mem - start));
+    return mem;
+}
+
+static void
+trim_spaces(char *buf, char *from)
+{
+    while (from > buf) {
+	--from;
+	if (*from == ' ')
+	    *from = '\0';
+	else
+	    break;
     }
 }
 
-/* This function should initially be called with buf = NULL. If
- * an error occurs, the file descriptor is closed. Subsequent
- * calls with a closed descriptor will cause buf to be deallocated.
- * Therefore, always check buf after calling this to see if an error
- * occurred.
- */
-static int
-read_field (const pam_handle_t *pamh, int fd, char **buf, int *from, int *to)
-{
-    /* is buf set ? */
+#define STATE_NL       0 /* new line starting */
+#define STATE_COMMENT  1 /* inside comment */
+#define STATE_FIELD    2 /* field following */
+#define STATE_EOF      3 /* end of file or error */
 
+static int
+read_field(const pam_handle_t *pamh, int fd, char **buf, int *from, int *state)
+{
+    char *to;
+    char *src;
+    int i;
+    char c;
+    int onspace;
+
+    /* is buf set ? */
     if (! *buf) {
-	*buf = (char *) malloc(PAM_GROUP_BUFLEN);
+	*buf = (char *) calloc(1, PAM_GROUP_BUFLEN+1);
 	if (! *buf) {
 	    pam_syslog(pamh, LOG_ERR, "out of memory");
+	    D(("no memory"));
+	    *state = STATE_EOF;
 	    return -1;
 	}
-	*from = *to = 0;
+	*from = 0;
+        *state = STATE_NL;
 	fd = open(PAM_GROUP_CONF, O_RDONLY);
+	if (fd < 0) {
+	    pam_syslog(pamh, LOG_ERR, "error opening %s: %m", PAM_GROUP_CONF);
+	    _pam_drop(*buf);
+	    *state = STATE_EOF;
+	    return -1;
+	}
     }
 
-    /* do we have a file open ? return error */
+    if (*from > 0)
+	to = shift_buf(*buf, *from);
+    else
+	to = *buf;
 
-    if (fd < 0 && *to <= 0) {
-	pam_syslog(pamh, LOG_ERR, "%s not opened", PAM_GROUP_CONF);
-	memset(*buf, 0, PAM_GROUP_BUFLEN);
-	_pam_drop(*buf);
-	return -1;
-    }
-
-    /* check if there was a newline last time */
-
-    if ((*to > *from) && (*to > 0)
-	&& ((*buf)[*from] == '\0')) {   /* previous line ended */
-	(*from)++;
-	(*buf)[0] = '\0';
-	return fd;
-    }
-
-    /* ready for more data: first shift the buffer's remaining data */
-
-    *to -= *from;
-    shift_bytes(*buf, *from, *to);
-    *from = 0;
-    (*buf)[*to] = '\0';
-
-    while (fd >= 0 && *to < PAM_GROUP_BUFLEN) {
-	int i;
-
-	/* now try to fill the remainder of the buffer */
-
-	i = read(fd, *to + *buf, PAM_GROUP_BUFLEN - *to);
+    while (fd != -1 && to - *buf < PAM_GROUP_BUFLEN) {
+	i = pam_modutil_read(fd, to, PAM_GROUP_BUFLEN - (to - *buf));
 	if (i < 0) {
 	    pam_syslog(pamh, LOG_ERR, "error reading %s: %m", PAM_GROUP_CONF);
 	    close(fd);
+	    memset(*buf, 0, PAM_GROUP_BUFLEN);
+	    _pam_drop(*buf);
+	    *state = STATE_EOF;
 	    return -1;
 	} else if (!i) {
 	    close(fd);
 	    fd = -1;          /* end of file reached */
-	} else
-	    *to += i;
-
-	/*
-	 * contract the buffer. Delete any comments, and replace all
-	 * multiple spaces with single commas
-	 */
-
-	i = 0;
-#ifdef DEBUG_DUMP
-	D(("buffer=<%s>",*buf));
-#endif
-	while (i < *to) {
-	    if ((*buf)[i] == ',') {
-		int j;
-
-		for (j=++i; j<*to && (*buf)[j] == ','; ++j);
-		if (j!=i) {
-		    shift_bytes(i + (*buf), j-i, (*to) - j);
-		    *to -= j-i;
-		}
-	    }
-	    switch ((*buf)[i]) {
-		int j, c;
-	    case '#':
-	        c = 0;
-		for (j=i; j < *to && (c = (*buf)[j]) != '\n'; ++j);
-		if (j >= *to) {
-		    (*buf)[*to = ++i] = '\0';
-		} else if (c == '\n') {
-		    shift_bytes(i + (*buf), j-i, (*to) - j);
-		    *to -= j-i;
-		    ++i;
-		} else {
-		    pam_syslog(pamh, LOG_CRIT,
-			       "internal error in file %s at line %d",
-			       __FILE__, __LINE__);
-	            close(fd);
-		    return -1;
-		}
-		break;
-	    case '\\':
-		if ((*buf)[i+1] == '\n') {
-		    shift_bytes(i + *buf, 2, *to - (i+2));
-		    *to -= 2;
-		} else {
-		    ++i;   /* we don't escape non-newline characters */
-		}
-		break;
-	    case '!':
-	    case ' ':
-	    case '\t':
-		if ((*buf)[i] != '!')
-		    (*buf)[i] = ',';
-		/* delete any trailing spaces */
-		for (j=++i; j < *to && ( (c = (*buf)[j]) == ' '
-					 || c == '\t' ); ++j);
-		shift_bytes(i + *buf, j-i, (*to)-j );
-		*to -= j-i;
-		break;
-	    default:
-		++i;
-	    }
 	}
+
+	to += i;
     }
 
-    (*buf)[*to] = '\0';
+    if (to == *buf) {
+	/* nothing previously in buf, nothing read */
+	_pam_drop(*buf);
+	*state = STATE_EOF;
+	return -1;
+    }
 
-    /* now return the next field (set the from/to markers) */
-    {
-	int i;
+    memset(to, '\0', PAM_GROUP_BUFLEN - (to - *buf));
 
-	for (i=0; i<*to; ++i) {
-	    switch ((*buf)[i]) {
-	    case '#':
-	    case '\n':               /* end of the line/file */
-		(*buf)[i] = '\0';
-		*from = i;
+    to = *buf;
+    onspace = 1; /* delete any leading spaces */
+
+    for (src = to; (c=*src) != '\0'; ++src) {
+	if (*state == STATE_COMMENT && c != '\n') {
+		continue;
+	}
+
+	switch (c) {
+	    case '\n':
+		*state = STATE_NL;
+                *to = '\0';
+		*from = (src - *buf) + 1;
+		trim_spaces(*buf, to);
 		return fd;
-	    case FIELD_SEPARATOR:    /* end of the field */
-		(*buf)[i] = '\0';
-	    *from = ++i;
-	    return fd;
-	    }
+
+	    case '\t':
+            case ' ':
+		if (!onspace) {
+		    onspace = 1;
+		    *to++ = ' ';
+		}
+		break;
+
+            case '!':
+		onspace = 1; /* ignore following spaces */
+		*to++ = '!';
+		break;
+
+	    case '#':
+		*state = STATE_COMMENT;
+		break;
+
+	    case FIELD_SEPARATOR:
+		*state = STATE_FIELD;
+                *to = '\0';
+		*from = (src - *buf) + 1;
+		trim_spaces(*buf, to);
+		return fd;
+
+	    case '\\':
+		if (src[1] == '\n') {
+		    ++src; /* skip it */
+		    break;
+		}
+	    default:
+		*to++ = c;
+		onspace = 0;
 	}
-	*from = i;
-	(*buf)[*from] = '\0';
+	if (src > to)
+	    *src = '\0'; /* clearing */
     }
 
-    if (*to <= 0) {
-	D(("[end of text]"));
-	*buf = NULL;
+    if (*state != STATE_COMMENT) {
+	*state = STATE_COMMENT;
+	pam_syslog(pamh, LOG_ERR, "field too long - ignored");
+	**buf = '\0';
+    } else {
+	*to = '\0';
+	trim_spaces(*buf, to);
     }
+
+    *from = 0;
     return fd;
 }
 
@@ -220,11 +208,10 @@ read_field (const pam_handle_t *pamh, int fd, char **buf, int *from, int *to)
 
 static int logic_member(const char *string, int *at)
 {
-     int len,c,to;
+     int c,to;
      int done=0;
      int token=0;
 
-     len=0;
      to=*at;
      do {
 	  c = string[to++];
@@ -263,13 +250,13 @@ static int logic_member(const char *string, int *at)
 
 typedef enum { VAL, OP } expect;
 
-static boolean
+static int
 logic_field (const pam_handle_t *pamh, const void *me,
              const char *x, int rule,
-             boolean (*agrees)(const pam_handle_t *pamh, const void *,
+             int (*agrees)(const pam_handle_t *pamh, const void *,
                                const char *, int, int))
 {
-     boolean left=FALSE, right, not=FALSE;
+     int left=FALSE, right, not=FALSE;
      operator oper=OR;
      int at=0, l;
      expect next=VAL;
@@ -280,7 +267,8 @@ logic_field (const pam_handle_t *pamh, const void *me,
 	  if (next == VAL) {
 	       if (c == '!')
 		    not = !not;
-	       else if (isalpha(c) || c == '*') {
+	       else if (isalpha(c) || c == '*' || isdigit(c) || c == '_'
+                    || c == '-' || c == '.' || c == '/' || c == ':') {
 		    right = not ^ agrees(pamh, me, x+at, l, rule);
 		    if (oper == AND)
 			 left &= right;
@@ -316,7 +304,7 @@ logic_field (const pam_handle_t *pamh, const void *me,
      return left;
 }
 
-static boolean
+static int
 is_same (const pam_handle_t *pamh UNUSED,
          const void *A, const char *b, int len, int rule UNUSED)
 {
@@ -332,6 +320,13 @@ is_same (const pam_handle_t *pamh UNUSED,
 		    return FALSE;
 	  }
      }
+
+     /* Ok, we know that b is a substring from A and does not contain
+        wildcards, but now the length of both strings must be the same,
+        too. In this case it means, a[i] has to be the end of the string. */
+     if (a[i] != '\0')
+          return FALSE;
+
      return ( !len );
 }
 
@@ -373,11 +368,11 @@ static TIME time_now(void)
 }
 
 /* take the current date and see if the range "date" passes it */
-static boolean
+static int
 check_time (const pam_handle_t *pamh, const void *AT,
             const char *times, int len, int rule)
 {
-     boolean not,pass;
+     int not,pass;
      int marked_day, time_start, time_end;
      const TIME *at;
      int i,j=0;
@@ -477,11 +472,10 @@ check_time (const pam_handle_t *pamh, const void *AT,
 
 static int find_member(const char *string, int *at)
 {
-     int len,c,to;
+     int c,to;
      int done=0;
      int token=0;
 
-     len=0;
      to=*at;
      do {
           c = string[to++];
@@ -579,7 +573,7 @@ static int mkgrplist(pam_handle_t *pamh, char *buf, gid_t **list, int len)
 static int check_account(pam_handle_t *pamh, const char *service,
 			 const char *tty, const char *user)
 {
-    int from=0,to=0,fd=-1;
+    int from=0, state=STATE_NL, fd=-1;
     char *buffer=NULL;
     int count=0;
     TIME here_and_now;
@@ -600,9 +594,9 @@ static int check_account(pam_handle_t *pamh, const char *service,
 	if (getgroups(no_grps, grps) < 0) {
 	    D(("getgroups call failed"));
 	    no_grps = 0;
-	    grps = NULL;
+	    _pam_drop(grps);
 	}
-#ifdef DEBUG
+#ifdef PAM_DEBUG
 	{
 	    int z;
 	    for (z=0; z<no_grps; ++z) {
@@ -624,7 +618,7 @@ static int check_account(pam_handle_t *pamh, const char *service,
 
 	/* here we get the service name field */
 
-	fd = read_field(pamh,fd,&buffer,&from,&to);
+	fd = read_field(pamh, fd, &buffer, &from, &state);
 	if (!buffer || !buffer[0]) {
 	    /* empty line .. ? */
 	    continue;
@@ -632,15 +626,21 @@ static int check_account(pam_handle_t *pamh, const char *service,
 	++count;
 	D(("working on rule #%d",count));
 
+	if (state != STATE_FIELD) {
+	    pam_syslog(pamh, LOG_ERR,
+		       "%s: malformed rule #%d", PAM_GROUP_CONF, count);
+	    continue;
+	}
+
 	good = logic_field(pamh,service, buffer, count, is_same);
 	D(("with service: %s", good ? "passes":"fails" ));
 
 	/* here we get the terminal name field */
 
-	fd = read_field(pamh,fd,&buffer,&from,&to);
-	if (!buffer || !buffer[0]) {
+	fd = read_field(pamh, fd, &buffer, &from, &state);
+	if (state != STATE_FIELD) {
 	    pam_syslog(pamh, LOG_ERR,
-		       "%s: no tty entry #%d", PAM_GROUP_CONF, count);
+		       "%s: malformed rule #%d", PAM_GROUP_CONF, count);
 	    continue;
 	}
 	good &= logic_field(pamh,tty, buffer, count, is_same);
@@ -648,35 +648,42 @@ static int check_account(pam_handle_t *pamh, const char *service,
 
 	/* here we get the username field */
 
-	fd = read_field(pamh,fd,&buffer,&from,&to);
-	if (!buffer || !buffer[0]) {
+	fd = read_field(pamh, fd, &buffer, &from, &state);
+	if (state != STATE_FIELD) {
 	    pam_syslog(pamh, LOG_ERR,
-		       "%s: no user entry #%d", PAM_GROUP_CONF, count);
+		       "%s: malformed rule #%d", PAM_GROUP_CONF, count);
 	    continue;
 	}
 	/* If buffer starts with @, we are using netgroups */
 	if (buffer[0] == '@')
+#ifdef HAVE_INNETGR
 	  good &= innetgr (&buffer[1], NULL, user, NULL);
+#else
+	  pam_syslog (pamh, LOG_ERR, "pam_group does not have netgroup support");
+#endif
+	/* otherwise, if the buffer starts with %, it's a UNIX group */
+	else if (buffer[0] == '%')
+          good &= pam_modutil_user_in_group_nam_nam(pamh, user, &buffer[1]);
 	else
 	  good &= logic_field(pamh,user, buffer, count, is_same);
 	D(("with user: %s", good ? "passes":"fails" ));
 
 	/* here we get the time field */
 
-	fd = read_field(pamh,fd,&buffer,&from,&to);
-	if (!buffer || !buffer[0]) {
+	fd = read_field(pamh, fd, &buffer, &from, &state);
+	if (state != STATE_FIELD) {
 	    pam_syslog(pamh, LOG_ERR,
-		       "%s: no time entry #%d", PAM_GROUP_CONF, count);
+		       "%s: malformed rule #%d", PAM_GROUP_CONF, count);
 	    continue;
 	}
 
 	good &= logic_field(pamh,&here_and_now, buffer, count, check_time);
 	D(("with time: %s", good ? "passes":"fails" ));
 
-	fd = read_field(pamh,fd,&buffer,&from,&to);
-	if (!buffer || !buffer[0]) {
+	fd = read_field(pamh, fd, &buffer, &from, &state);
+	if (state == STATE_FIELD) {
 	    pam_syslog(pamh, LOG_ERR,
-		       "%s: no listed groups for rule #%d", PAM_GROUP_CONF, count);
+		       "%s: poorly terminated rule #%d", PAM_GROUP_CONF, count);
 	    continue;
 	}
 
@@ -695,14 +702,6 @@ static int check_account(pam_handle_t *pamh, const char *service,
 	    }
 	}
 
-	/* check the line is terminated correctly */
-
-	fd = read_field(pamh,fd,&buffer,&from,&to);
-	if (buffer && buffer[0]) {
-	    pam_syslog(pamh, LOG_ERR,
-                       "%s: poorly terminated rule #%d", PAM_GROUP_CONF, count);
-	}
-
 	if (good > 0) {
 	    D(("rule #%d passed, added %d groups", count, good));
 	} else if (good < 0) {
@@ -711,20 +710,22 @@ static int check_account(pam_handle_t *pamh, const char *service,
 	    D(("rule #%d failed", count));
 	}
 
-    } while (buffer);
+    } while (state != STATE_EOF);
 
     /* now set the groups for the user */
 
     if (no_grps > 0) {
+#ifdef PAM_DEBUG
 	int err;
+#endif
 	D(("trying to set %d groups", no_grps));
-#ifdef DEBUG
+#ifdef PAM_DEBUG
 	for (err=0; err<no_grps; ++err) {
 	    D(("gid[%d]=%d", err, grps[err]));
 	}
 #endif
-	if ((err = setgroups(no_grps, grps))) {
-	    D(("but couldn't set groups %d", err));
+	if (setgroups(no_grps, grps) != 0) {
+	    D(("but couldn't set groups %m"));
 	    pam_syslog(pamh, LOG_ERR,
 		       "unable to set the group membership for user: %m");
 	    retval = PAM_CRED_ERR;
@@ -742,14 +743,14 @@ static int check_account(pam_handle_t *pamh, const char *service,
 
 /* --- public authentication management functions --- */
 
-PAM_EXTERN int
+int
 pam_sm_authenticate (pam_handle_t *pamh UNUSED, int flags UNUSED,
 		     int argc UNUSED, const char **argv UNUSED)
 {
     return PAM_IGNORE;
 }
 
-PAM_EXTERN int
+int
 pam_sm_setcred (pam_handle_t *pamh, int flags,
 		int argc UNUSED, const char **argv UNUSED)
 {
@@ -820,18 +821,3 @@ pam_sm_setcred (pam_handle_t *pamh, int flags,
 }
 
 /* end of module definition */
-
-#ifdef PAM_STATIC
-
-/* static module data */
-
-struct pam_module _pam_group_modstruct = {
-    "pam_group",
-    pam_sm_authenticate,
-    pam_sm_setcred,
-    NULL,
-    NULL,
-    NULL,
-    NULL
-};
-#endif
