@@ -276,12 +276,12 @@ last_login_read(pam_handle_t *pamh, int announce, int last_fd, uid_t uid, time_t
 		time_t ll_time;
 
 		ll_time = last_login.ll_time;
-		tm = localtime_r (&ll_time, &tm_buf);
-		strftime (the_time, sizeof (the_time),
-	        /* TRANSLATORS: "strftime options for date of last login" */
-			  _(" %a %b %e %H:%M:%S %Z %Y"), tm);
-
-		date = the_time;
+		if ((tm = localtime_r (&ll_time, &tm_buf)) != NULL) {
+			strftime (the_time, sizeof (the_time),
+		        /* TRANSLATORS: "strftime options for date of last login" */
+				  _(" %a %b %e %H:%M:%S %Z %Y"), tm);
+			date = the_time;
+		}
 	    }
 
 	    /* we want & have the host? */
@@ -566,7 +566,7 @@ cleanup:
 }
 
 /* --- authentication (locking out inactive users) functions --- */
-PAM_EXTERN int
+int
 pam_sm_authenticate(pam_handle_t *pamh, int flags,
 		    int argc, const char **argv)
 {
@@ -636,14 +636,14 @@ pam_sm_authenticate(pam_handle_t *pamh, int flags,
     return PAM_SUCCESS;
 }
 
-PAM_EXTERN int
+int
 pam_sm_setcred(pam_handle_t *pamh UNUSED, int flags UNUSED,
 		    int argc UNUSED, const char **argv UNUSED)
 {
     return PAM_SUCCESS;
 }
 
-PAM_EXTERN int
+int
 pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
 		    int argc, const char **argv)
 {
@@ -652,7 +652,7 @@ pam_sm_acct_mgmt(pam_handle_t *pamh, int flags,
 
 /* --- session management functions --- */
 
-PAM_EXTERN int
+int
 pam_sm_open_session(pam_handle_t *pamh, int flags,
 		    int argc, const char **argv)
 {
@@ -702,7 +702,7 @@ pam_sm_open_session(pam_handle_t *pamh, int flags,
     return retval;
 }
 
-PAM_EXTERN int
+int
 pam_sm_close_session (pam_handle_t *pamh, int flags,
 		      int argc, const char **argv)
 {
@@ -719,20 +719,4 @@ pam_sm_close_session (pam_handle_t *pamh, int flags,
     return PAM_SUCCESS;
 }
 
-#ifdef PAM_STATIC
-
-/* static module data */
-
-struct pam_module _pam_lastlog_modstruct = {
-     "pam_lastlog",
-     pam_sm_authenticate,
-     pam_sm_setcred,
-     pam_sm_acct_mgmt,
-     pam_sm_open_session,
-     pam_sm_close_session,
-     NULL,
-};
-
-#endif
-
 /* end of module definition */
