@@ -31,7 +31,7 @@
 
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
-#define SELINUX_ENABLED is_selinux_enabled()>0
+#define SELINUX_ENABLED (is_selinux_enabled()>0)
 #else
 #define SELINUX_ENABLED 0
 #endif
@@ -417,12 +417,9 @@ PAMH_ARG_DECL(char * create_password_hash,
 #endif
 		sp = stpcpy(salt, algoid);
 		if (on(UNIX_ALGO_ROUNDS, ctrl)) {
-			sp += snprintf(sp, sizeof(salt) - 3, "rounds=%u$", rounds);
+			sp += snprintf(sp, sizeof(salt) - (16 + 1 + (sp - salt)), "rounds=%u$", rounds);
 		}
-		crypt_make_salt(sp, 8);
-		/* For now be conservative so the resulting hashes
-		 * are not too long. 8 bytes of salt prevents dictionary
-		 * attacks well enough. */
+		crypt_make_salt(sp, 16);
 #ifdef HAVE_CRYPT_GENSALT_R
 	}
 #endif
@@ -961,7 +958,7 @@ PAMH_ARG_DECL(int unix_update_shadow,
     fclose(opwfile);
 
     if (!wroteentry && !err) {
-	spwdent.sp_namp = forwho;
+	spwdent.sp_namp = (char *)forwho;
 	spwdent.sp_pwdp = towhat;
 	spwdent.sp_lstchg = time(NULL) / (60 * 60 * 24);
 	if (spwdent.sp_lstchg == 0)
